import json
import re
from collections import defaultdict

import pronto as pt

from ipc.settings import ROOT_DIR

search_for_disease, search_for_organism = 'MESH', 'NCBITaxon'
tissues = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/tissue.obo')
cell_types = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/cell_type.obo')
cell_line = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/cell_line.obo')
drugs = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/drug.obo')
diseases = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/disease.obo')


class SetEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, set):
            return list(obj)
        return json.JSONEncoder.default(self, obj)


all_names = {'tissue': set(),
             'kw_cell_type': set(),
             'kw_cell_line': dict(),
             'disease': set(),
             'kw_drug': set(),
             'organism': set()}
for term in tissues.terms():
    if 'tissue' in term.subsets:
        all_names['tissue'].add(term.name)

for term in cell_types.terms():
    all_names['kw_cell_type'].add(term.name)
for term in diseases.terms():
    all_names['disease'].add(term.name)
for term in drugs.terms():
    all_names['kw_drug'].add(term.name)
# for term in organisms.terms():
#     all_names['organsisms'].add(term.name)

organism_file = open(ROOT_DIR.parent / 'knowledge_base' / 'ncbi_names.psv', 'r')
while organism_file:
    line = organism_file.readline()
    if not line:
        break
    all_names['organism'].add(line.split('|')[1].strip())
organism_file.close()

all_names["kw_data_type"] = ["Transcriptomics",
                             "Metabolomics",
                             "Mutation",
                             "MiRNA",
                             "Proteomics",
                             "Single cell",
                             "Gene dependency",
                             "Methylation",
                             "Drug Screens",
                             "Gene effect",
                             "RNAi",
                             "Copy Number Variation",
                             "Cytometry",
                             "Lab measurement",
                             "PCR",
                             "Titer",
                             "Fusion"]

all_names['dataset_source'] = ["GEO",
                               "cBioportal",
                               "TCGA",
                               "Metabolomics Workbench",
                               "DepMap",
                               "PharmacoDB",
                               "GDC",
                               "CPTAC",
                               "CCLE",
                               "LINCS",
                               "PPMI",
                               "HPA",
                               "MetaboLights",
                               "GTEx",
                               "ArrayExpress",
                               "SingleCellPortal",
                               "Figshare",
                               "HCA",
                               "Immport",
                               "Zenodo",
                               "Publication",
                               "gnomAD",
                               "RCSB",
                               "SynapseAuron",
                               "Cygnal",
                               "NGJ",
                               "RubiusTx",
                               "Sigilon",
                               "Yale",
                               "Valo"]

cell_line_disease_dict = defaultdict(set)
for term in cell_line.terms():
    search_res = [item.description for item in term.xrefs if re.search(search_for_disease, item.id)]
    if search_res:
        cell_line_disease_dict[search_res[0]].add(term.name)

cell_line_organism_dict = defaultdict(set)
for term in cell_line.terms():
    search_res = [item.description for item in term.xrefs if re.search(search_for_organism, item.id)]
    if search_res:
        cell_line_organism_dict[search_res[0]].add(term.name)

valid_cell_line_names = []
for term in cell_line.terms():
    valid_cell_line_names.append(term.name)
all_names['kw_cell_line'] = {'valid_cell_line_names': valid_cell_line_names,
                             'organism': cell_line_organism_dict,
                             'disease': cell_line_disease_dict}

with open(ROOT_DIR / "ipc" / "valid_names.json", 'w') as openfile:
    json.dump(all_names, openfile, cls=SetEncoder, indent=4)
